const { prompt } = require('enquirer');
const data = require('../utils/data');
const { dim } = require('chalk');
const alert = require('cli-alerts');

async function selectProjectForm(
	projects,
	cancelOption = true,
	question = 'Please select the project'
) {
	let options = projects.map(item => {
		return {
			uuid: item.uuid,
			name: item.name
		};
	});

	if (cancelOption) {
		options.push({
			uuid: 'cancel',
			name: 'Cancel'
		});
	}

	return await prompt({
		type: 'select',
		name: 'project',
		message: question,
		choices: options,
		result: function () {
			return { name: this.focused.name, uuid: this.focused.uuid };
		}
	});
}

async function selectProjectMember(question) {
	let projects = await data.getProjects();
	const selectProject = await selectProjectForm(
		projects,
		true,
		`${question} \n ${dim('Please Select Project')}`
	);
	if (selectProject.project.uuid === 'cancel') process.exit();
	let memberList = projects.filter(
		project => project.uuid === selectProject.project.uuid
	)[0].members;
	if (memberList == null || memberList.length === 0) {
		alert({
			type: 'error',
			msg: `There are no members in the project ${selectProject.project.name}`
		});
		process.exit();
	}
	return memberList;
}

const confirmToggle = async () => {
	return await prompt({
		type: 'toggle',
		name: 'sure',
		message: 'Are you sure?',
		enabled: 'Yes',
		disabled: 'No'
	});
};

module.exports = {
	selectProjectForm,
	selectProjectMember,
	confirmToggle
};
