#!/usr/bin/env node

/**
 * elm-helper-cli
 * Representative hepler, to help you in meetings MOM. sprint notes, and other stuff
 *
 * @author Abdulrhman Alfulayt <https://twitter.com/Abdualrhman>
 */

const init = require('./utils/init');
const cli = require('./utils/cli');
const log = require('./utils/log');
const data = require('./utils/data');
const event = require('./utils/event');
const { meeting_mom } = require('./actions_operations/meeting_mom');
const { Select } = require('enquirer');
const alert = require('cli-alerts');
const { green: g, yellow: y, dim: d, bold: b } = require('chalk');
const {
	projectMainMenu,
	addNewProject
} = require('./actions_operations/projects');
const { addUserInfo } = require('./actions_operations/user_info');
const ora = require('ora');
const spinner = ora({ text: '' });

const input = cli.input;
const flags = cli.flags;
const { clear, debug } = flags;
const print = console.log;
const actionOptions = {
	KpiEvidence: `🗒  ${y('Generate')} ${b('KPI Evidence')} `,
	SprintMeetingNotes: `🗒  ${y('Generate')} ${b('Sprint Meeting Notes')} `,
	MeetingMom: `🔖 ${y('Generate')} ${b('Meeting MOM')}`,
	Converters: `🔁 ${b('Converters')}`,
	projectsSetting: '📦 Projects Setting',
	exit: '✖️ Exit'
};

function newProjectAddedSuccessfully(newProjectResult) {
	return (
		newProjectResult.success && {
			type: 'success',
			msg: `Project ${newProjectResult.name} added successfully`
		}
	);
}

const checkProjects = async () => {
	alert({
		type: 'info',
		msg: 'No projects found, you need to add your first project'
	});
	let newProjectResult = await addNewProject();
	let args = newProjectAddedSuccessfully(newProjectResult);
	return homePage(args);
};

const mainMenu = async () => {
	const prompt = new Select({
		name: 'action',
		message: 'What you want to do?',
		choices: Object.values(actionOptions)
	});

	prompt
		.run()
		.then(async answer => {
			switch (answer) {
				case actionOptions.KpiEvidence:
					let kpiResult = await meeting_mom('Kpi.json');
					return homePage({
						type: 'success',
						msg: kpiResult.message
					});
					break;

				case actionOptions.SprintMeetingNotes:
					await meeting_mom('meeting_mom.json');
					break;

				case actionOptions.MeetingMom:
					await meeting_mom('meeting_mom.json');
					break;

				case actionOptions.Converters:
					print('Converters');
					break;

				case actionOptions.projectsSetting:
					let projectResult = await projectMainMenu();
					print(projectResult);
					let args = null;
					switch (projectResult?.action) {
						case event.addNewProject:
							args = newProjectAddedSuccessfully(projectResult);
							break;
						case event.deleteProject:
							args = projectResult.success && {
								type: 'success',
								msg: `Project deleted successfully`
							};
							break;
						case event.addNewMember:
							args = projectResult.success && {
								type: 'success',
								msg: `${projectResult.name} added successfully to project ${projectResult.project}`
							};
							break;
						case event.deleteMember:
							args = projectResult.success
								? {
										type: 'success',
										msg: `${projectResult.name} deleted successfully from project ${projectResult.project}`
								  }
								: projectResult?.showMsg !== '' && {
										type: 'error',
										msg: projectResult?.showMsg
								  };
							break;
					}
					return homePage(args);
			}
		})
		.catch(console.error);
};

const homePage = async (args = null) => {
	init({ clear });
	input.includes(`help`) && cli.showHelp(0);
	if (args?.type != null) {
		alert({
			type: args.type,
			msg: args.msg
		});
	}

	let mainUser = await data.getMainUser();
	if (mainUser == null) {
		await addUserInfo();
		return;
	}

	// let projects = await data.getProjects();
	// if (!projects || projects.length === 0) {
	// 	await checkProjects();
	// 	return;
	// }

	await mainMenu();
	debug && log(flags);
};

(async () => {
	await homePage();
})();

module.exports = {
	homePage
};
