const axios = require('axios');
const { getMainUser } = require('../utils/data');
const cli = require('../utils/cli');

let WIKI_ENDPOINT = 'https://wiki.elm.sa/rest/api/';
const { debug, output } = cli.flags;
// Get all pages

class ApiService {
	username;
	password;
	constructor() {}

	async initialize() {
		let userData = await getMainUser();
		this.username = userData.username;
		this.password = userData.password;
		debug && console.log(userData);
		this.axios = axios.create({
			baseURL: WIKI_ENDPOINT,
			headers: {
				'Content-Type': 'application/json',
				Accept: 'application/json',
				Authorization:
					'Basic ' +
					Buffer.from(`${this.username}:${this.password}`).toString(
						'base64'
					)
			}
		});
	}

	static async create() {
		const api = new ApiService();
		await api.initialize();
		return api;
	}

	async getPageId(page_title) {
		let encodedString = encodeURIComponent(page_title);
		let response = await this.axios.get(
			`content?title=${encodedString}&spaceKey=~${this.username}`
		);
		debug && console.log(response.data);
		return response.data.results.length > 0
			? response.data.results[0].id
			: null;
	}

	async createPage(page_title, data) {
		let response = await this.axios.post(`content`, {
			type: 'page',
			title: page_title,
			space: {
				key: `~${this.username}`
			},
			body: {
				storage: {
					value: data,
					representation: 'storage'
				}
			}
		});

		debug && console.log(response.data);
		debug && console.log('page id : ' + response.data.id);
		return response.data;
	}

	async getPageContent(page_title, data) {
		console.log(page_title);
		let pageId;
		pageId = await this.getPageId(page_title);
		if (pageId === null) {
			let newPage = await this.createPage(page_title, data);
			pageId = newPage.id;
		}
		let response = await this.axios.get(
			`content/${pageId}?expand=body.storage,version`
		);
		let version = response.data.version.number;
		debug && console.log(response.data);
		return {
			content: response.data.body.storage.value,
			id: pageId,
			version
		};
	}

	async updatePage(id, title, content, version) {
		console.log(id);
		let response = await this.axios.put(`content/${id}`, {
			type: 'page',
			id: id,
			title: title,
			space: {
				key: `~${this.username}`
			},
			body: {
				storage: {
					value: content,
					representation: 'storage'
				}
			},
			version: { number: version }
		});
		debug && console.log(response.data);
		return response.data;
	}
}
//
// (async () => {
// 	debug = true;
// 	let apiService = await ApiService.create();
// 	let initSections = {
// 		builds: [{ date: '', URL: '', note: '' }],
// 		wiki_updates: [{ date: '', URL: '', note: '' }],
// 		initiative: [{ date: '', URL: '', note: '' }],
// 		reusable_components: [{ date: '', URL: '', note: '' }],
// 		technical_enhancements: [{ date: '', URL: '', note: '' }],
// 		other: [{ date: '', URL: '', note: '' }]
// 	};
//
// 	// let inis =
//
// 	let initData = page.convertObjectToHtml(initSections);
// 	let pageContent = await apiService.getPageContent('KPI 2022-Q3', initData);
//
// 	let data = page.convertHtmlToObject(pageContent);
//
// 	let ttt = {
// 		date: 'w-d3',
// 		URL: 'w-u1',
// 		note: 'w-n1'
// 	};
//
// 	data.builds.push(ttt);
//
// 	let newPageContent = page.convertObjectToHtml(data);
// 	console.log(newPageContent);
// })();

module.exports = ApiService;
