const fs = require('fs');
const { prompt } = require('enquirer');
const { yellow, dim, gray, green, bgGray, italic, cyan } = require('chalk');
const Helper = require('./helpers');
const { selectProjectMember } = require('../actions_operations/common');
const datePrompt = require('date-prompt');
const cli = require('../utils/cli');
const { debug } = cli.flags;

const generate_questions = async json_file => {
	let questions = [];
	let rawData = fs.readFileSync(json_file);
	let questions_json = await JSON.parse(rawData).questions;
	let questions_number = questions_json.length;

	for (let i = 0; i < questions_number; i++) {
		let question = {
			type: questions_json[i].type,
			name: questions_json[i].name,
			title: questions_json[i].title,
			message: questions_json[i].message,
			report_output_type: questions_json[i].report_output_type,
			data_source: questions_json[i].data_source,
			reAsking: questions_json[i].reAsking,
			choices: questions_json[i].choices,
			multiple_choice: questions_json[i].multiple_choice,
			required: questions_json[i].required
		};
		questions.push(question);
	}

	return questions;
};

async function dateQuestion(question, answers) {
	let answerObject = {};
	answerObject.name = question.name;
	answerObject.title = question.title;
	answerObject.report_output_type = question.report_output_type;
	await datePrompt(question.title)
		.then(isoStr => {
			let date = new Date(isoStr);
			console.log(
				`${dim(question.title + ' · ')}${cyan(date.toLocaleString())}`
			);
			answerObject.answer = date.toLocaleString();
			answers.push(answerObject);
		})
		.catch(isoStr => console.log('Aborted with', isoStr));
}

async function inputQuestion(question, answers) {
	let answerObject = {};
	let question_answer = [];
	answerObject.name = question.name;
	answerObject.title = question.title;
	answerObject.report_output_type = question.report_output_type;
	let last_answer = '';
	while (last_answer !== 'next') {
		let answer = await prompt({
			type: question.type,
			name: question.name,
			message: `${yellow(question.title)} \n ${dim(question.message)}  ${
				question.reAsking
					? dim(
							green(
								italic('(to goto next question, write "next")')
							)
					  )
					: ''
			} \n`,
			validate: question.required ? Helper.getValidate() : null,
			result: function (value) {
				return {
					name: this.name,
					value: value
				};
			}
		});
		if (answer[question.name].value !== 'next')
			question_answer.push(answer[question.name].value);

		last_answer = question.reAsking ? '' : 'next';
		if (last_answer === 'next') break;

		last_answer = answer[question.name].value;
	}
	answerObject.answer = question_answer;
	answers.push(answerObject);
}

async function toggleQuestion(question, answers) {
	let answerObject = {};
	answerObject.name = question.name;
	answerObject.title = question.title;
	answerObject.report_output_type = question.report_output_type;
	let answer = await prompt({
		type: question.type,
		name: question.name,
		message: `${yellow(question.title)} \n ${dim(question.message)} \n`
	});
	answerObject.answer = answer[question.name];
	answers.push(answerObject);
}

async function selectQuestion(question, answers) {
	let answerObject = {};
	answerObject.name = question.name;
	answerObject.title = question.title;
	answerObject.report_output_type = question.report_output_type;
	let choices = [];
	let fromProjectMembers = false;
	if (question.choices === 'from_data_source') {
		let data_source = question.data_source.split('.');
		fromProjectMembers =
			data_source[0] === 'project' && data_source[1] === 'members';
		if (fromProjectMembers) {
			let memberList = await selectProjectMember(question.title);
			choices = memberList.map(member => {
				return {
					name: member.username,
					message: member.name
				};
			});
		}
	} else {
		choices = question.choices;
	}
	debug && console.log(choices);
	let answer = await prompt({
		type: question.type,
		name: question.name,
		multiple: question.multiple_choice,
		message: `${yellow(question.title)} \n ${dim(question.message)}`,
		choices: choices,
		validate: question.required ? Helper.getValidate() : null
	});
	let answers_values = answer[question.name];

	if (fromProjectMembers) {
		let answers_names = choices.filter(choice => {
			return answers_values.includes(choice.value);
		});
		answerObject.answer = answers_names.map(answer => {
			return {
				key: answer.message,
				value: answer.name
			};
		});
	} else {
		answerObject.answer = answers_values;
	}

	answers.push(answerObject);
}

module.exports = {
	generate_questions,
	dateQuestion,
	inputQuestion,
	toggleQuestion,
	selectQuestion
};
