const fs = require('fs');
const execa = require('execa');
const cli = require('../utils/cli');
const os = require('os');
const path = require('path');
const ora = require('ora');
const { green } = require('chalk');
const j2m = require('jira2md');
const { mdToPdf } = require('md-to-pdf');
const helper = require('./helpers');
const {
	keyListMarkup,
	keyValueMarkup,
	titleMarkup
} = require('./markup_generator');
const ApiService = require('../services/api_services');
const page = require('./wiki/pages');
const spinner = ora({ text: '' });

const { debug, output } = cli.flags;

const reportOutputType = {
	pdf: 'pdf',
	markdown: 'markdown',
	excel: 'excel'
};

const outputDataType = {
	text: 'text',
	point_list: 'point_list',
	list_numbers: 'list_numbers',
	title: 'title'
};
let finalOutput = '';
let outputObject = {};
const generateReport = async (answers, reportSchemaPath) => {
	let OutputType = output;
	debug && console.log(`report type is ${OutputType}`);
	spinner.start(`Generating report in ${OutputType} format...`);
	let rawData = fs.readFileSync(reportSchemaPath);
	let reportSchema = await JSON.parse(rawData);
	let schema_json = reportSchema.output_schema;
	await generateReportContent(answers, schema_json, OutputType);
	if (reportSchema.generate_to === 'file') {
		return await generateReportFile(OutputType);
	} else {
		let searchKeys = reportSchema.init_file_choices;
		let init_wiki_choice = getQuestionChoices(searchKeys, reportSchema);
		return await generateAndSendReport(init_wiki_choice, searchKeys);
	}
};

const getQuestionChoices = (searchKeys, reportSchema) => {
	let initialSchema = helper
		.searchTest(searchKeys, reportSchema)
		.filter(item => item.parent.choices !== undefined)[0].parent;
	return initialSchema.choices;
};

// (async () => {
// 	console.log('fff');
// 	let path_to_questions = path.join(__dirname, '../questions/kpi.json');
// 	let rawData = fs.readFileSync(path_to_questions);
// 	let reportSchema = await JSON.parse(rawData);
// 	let searchKeys = reportSchema.init_file_choices;
// 	let init_wiki_choice = getQuestionChoices(searchKeys, reportSchema);
//
// 	let dor = {};
// 	init_wiki_choice.forEach(item => {
// 		let value = item.value.replace(/ /g, '_').toLowerCase();
// 		let sectionData = {
// 			[value]: [
// 				{
// 					date: '',
// 					URL: '',
// 					note: ''
// 				}
// 			]
// 		};
// 		dor = { ...dor, ...sectionData };
// 	});
//
// 	console.log(dor);
// })();

const generateReportContent = async (answers, reportSchema, reportType) => {
	return await new Promise(async (resolve, reject) => {
		let schemaLength = reportSchema.length;
		for (let i = 0; i < schemaLength; i++) {
			let output = reportSchema[i];
			let outputType = output.type;

			let dataSourceKey = output.data_source_name;
			let text = output.text;
			let isValueOnly =
				reportType === reportOutputType.markdown
					? output.value_only
					: false;
			let isUsernames =
				reportType === reportOutputType.markdown
					? output.usernames
					: false;
			let newLine = output.new_line;
			debug && console.log(output);
			switch (outputType) {
				case outputDataType.title:
					answers
						.filter(answer => answer.name === dataSourceKey)
						.forEach(answer => {
							if (reportType === reportOutputType.excel) {
								keyValueExcel(text, answer.answer, newLine);
							} else {
								finalOutput += titleMarkup(text, answer.answer);
							}
						});
					break;
				case outputDataType.text:
					answers
						.filter(answer => answer.name === dataSourceKey)
						.forEach(answer => {
							if (reportType === reportOutputType.excel) {
								keyValueExcel(text, answer.answer, newLine);
							} else {
								let type = text
									.replace(/ /g, '_')
									.toLowerCase();
								let ans = {
									[type]: Array.isArray(answer.answer)
										? answer.answer[0]
										: answer.answer
								};
								outputObject = {
									...outputObject,
									...ans
								};
								finalOutput += keyValueMarkup(
									text,
									answer.answer,
									newLine
								);
							}
						});
					break;
				case outputDataType.point_list:
					answers
						.filter(answer => answer.name === dataSourceKey)
						.forEach(answer => {
							if (reportType === reportOutputType.excel) {
								keyListExcel(
									text,
									answer.answer,
									isValueOnly,
									isUsernames
								);
							} else {
								finalOutput += keyListMarkup(
									text,
									answer.answer,
									isValueOnly,
									isUsernames
								);
							}
						});
					break;
				case outputDataType.list_numbers:
					answers
						.filter(answer => answer.name === dataSourceKey)
						.forEach(answer => {
							if (reportType === reportOutputType.excel) {
								keyListExcel(
									text,
									answer.answer,
									isValueOnly,
									isUsernames,
									true
								);
							} else {
								finalOutput += keyListMarkup(
									text,
									answer.answer,
									isValueOnly,
									isUsernames,
									true
								);
							}
						});
					break;
			}
			finalOutput += '\n';
		}
		finalOutput += '\n';
		debug && console.log(finalOutput);
		resolve();
	});
};

const printer = value => {
	// finalOutput += '\n' + value;
	// debug && console.log(value);
};

const keyValueExcel = (key, value, newLine = false) => {
	// printer(`*${key}*${newLine ? ':\n' : ': '}${value}`);
};

const keyListExcel = (
	key,
	values,
	valueOnly = false,
	isUsernames = false,
	numbered = false
) => {
	// printer(`h3. ${key}:`);
	// values.forEach(value => {
	// 	printOutLists(numbered ? '#' : '-', value, isUsernames, valueOnly);
	// });
};

const getInitSchema = initialSchema => {
	let init = {};
	initialSchema.forEach(item => {
		let value = item.value.replace(/ /g, '_').toLowerCase();
		let sectionData = {
			[value]: [{}]
		};
		init = { ...init, ...sectionData };
	});
	return init;
};

const generateAndSendReport = async (initialSchema, searchKeys) => {
	const apiService = await ApiService.create();

	let currentQuarter = 'KPI ' + helper.getQuarter().join('-Q');

	let initSections = getInitSchema(initialSchema);
	let initData = page.convertObjectToHtml(initSections);
	let { content, id, version } = await apiService.getPageContent(
		currentQuarter,
		initData
	);
	let data = page.convertHtmlToObject(content);
	debug && console.log(outputObject);

	let dataSourceKey = outputObject[
		Object.keys(outputObject).filter(key => key === searchKeys)
	]
		.replace(/ /g, '_')
		.toLowerCase();

	let newObject = {
		date: new Date()
			.toLocaleString('en-CA', {
				year: 'numeric',
				month: '2-digit',
				day: '2-digit'
			})
			.replace(/\//g, '-'),
		URL: outputObject.evidence_url,
		note: outputObject.evidence_note
	};
	if (data[dataSourceKey]) {
		data[dataSourceKey].push(newObject);
	} else {
		data[dataSourceKey] = [newObject];
	}

	debug && console.log(data);

	await apiService.updatePage(
		id,
		currentQuarter,
		page.convertObjectToHtml(data),
		version + 1
	);

	spinner.succeed(
		`${green(`Report generated successfully, saved at your desktop.`)}`
	);

	return {
		status: 'success',
		message: 'Report generated and sent to your Wiki'
	};
};

const generateReportFile = async reportType => {
	const desktopPath = path.join(os.homedir(), 'Desktop');
	const date = new Date().toISOString().split('T')[0];
	const random = Math.floor(Math.random() * 1000);
	let fileData = { desktopPath, date, random };
	switch (reportType) {
		case reportOutputType.pdf:
			return await generatePdfReport(fileData);
		case reportOutputType.markdown:
			return await generateMarkdownReport(fileData);
		case reportOutputType.excel:
			return await generateExcelReport();
	}
};

const generatePdfReport = async fileData => {
	return await new Promise(async (resolve, reject) => {
		let md = j2m.to_markdown(finalOutput);
		let fileName = `${fileData.date}_${fileData.random}.pdf`;
		const finalPath = fileData.desktopPath + '/' + fileName;
		await mdToPdf(
			{ content: md },
			{
				dest: finalPath,
				pdf_options: {
					format: 'a4',
					printBackground: false,
					headerTemplate: helper.getPdfHeader(),
					footerTemplate: helper.getPdfFooter(),
					margin: {
						top: '40mm',
						left: '20mm',
						right: '20mm',
						bottom: '20mm'
					}
				}
			}
		);

		spinner.succeed(
			`${green(
				`Report ${fileName} generated successfully, saved at your desktop.`
			)}`
		);
		console.log();
		await execa('open', [finalPath]);
		resolve({
			status: 'success',
			message: `Report ${fileName} generated successfully, saved at your desktop.`
		});
	});
};

const generateMarkdownReport = async fileData => {
	return await new Promise(async (resolve, reject) => {
		let fileName = `${fileData.date}_${fileData.random}.text`;
		const finalPath = fileData.desktopPath + '/' + fileName;
		await fs.writeFile(finalPath, finalOutput, async err => {
			if (err) {
				reject(err);
				debug && console.error(err);
			} else {
				spinner.succeed(
					`${green(
						`Report ${fileName} generated successfully, saved at your desktop.`
					)}`
				);
				console.log();
				await execa('open', [finalPath]);
				resolve({
					status: 'success',
					message: `Report ${fileName} generated successfully, saved at your desktop.`
				});
			}
		});
	});
};

module.exports = {
	generateReport,
	reportOutputType,
	outputDataType
};
